XK_BackSpace = 0xFF08
XK_Tab = 0xFF09
XK_Linefeed = 0xFF0A
XK_Clear = 0xFF0B
XK_Return = 0xFF0D
XK_Pause = 0xFF13
XK_Scroll_Lock = 0xFF14
XK_Sys_Req = 0xFF15
XK_Escape = 0xFF1B
XK_Delete = 0xFFFF
XK_Multi_key = 0xFF20
XK_SingleCandidate = 0xFF3C
XK_MultipleCandidate = 0xFF3D
XK_PreviousCandidate = 0xFF3E
XK_Kanji = 0xFF21
XK_Muhenkan = 0xFF22
XK_Henkan_Mode = 0xFF23
XK_Henkan = 0xFF23
XK_Romaji = 0xFF24
XK_Hiragana = 0xFF25
XK_Katakana = 0xFF26
XK_Hiragana_Katakana = 0xFF27
XK_Zenkaku = 0xFF28
XK_Hankaku = 0xFF29
XK_Zenkaku_Hankaku = 0xFF2A
XK_Touroku = 0xFF2B
XK_Massyo = 0xFF2C
XK_Kana_Lock = 0xFF2D
XK_Kana_Shift = 0xFF2E
XK_Eisu_Shift = 0xFF2F
XK_Eisu_toggle = 0xFF30
XK_Zen_Koho = 0xFF3D
XK_Mae_Koho = 0xFF3E
XK_Home = 0xFF50
XK_Left = 0xFF51
XK_Up = 0xFF52
XK_Right = 0xFF53
XK_Down = 0xFF54
XK_Prior = 0xFF55
XK_Page_Up = 0xFF55
XK_Next = 0xFF56
XK_Page_Down = 0xFF56
XK_End = 0xFF57
XK_Begin = 0xFF58
XK_Select = 0xFF60
XK_Print = 0xFF61
XK_Execute = 0xFF62
XK_Insert = 0xFF63
XK_Undo = 0xFF65
XK_Redo = 0xFF66
XK_Menu = 0xFF67
XK_Find = 0xFF68
XK_Cancel = 0xFF69
XK_Help = 0xFF6A
XK_Break = 0xFF6B
XK_Mode_switch = 0xFF7E
XK_script_switch = 0xFF7E
XK_Num_Lock = 0xFF7F
XK_KP_Space = 0xFF80
XK_KP_Tab = 0xFF89
XK_KP_Enter = 0xFF8D
XK_KP_F1 = 0xFF91
XK_KP_F2 = 0xFF92
XK_KP_F3 = 0xFF93
XK_KP_F4 = 0xFF94
XK_KP_Home = 0xFF95
XK_KP_Left = 0xFF96
XK_KP_Up = 0xFF97
XK_KP_Right = 0xFF98
XK_KP_Down = 0xFF99
XK_KP_Prior = 0xFF9A
XK_KP_Page_Up = 0xFF9A
XK_KP_Next = 0xFF9B
XK_KP_Page_Down = 0xFF9B
XK_KP_End = 0xFF9C
XK_KP_Begin = 0xFF9D
XK_KP_Insert = 0xFF9E
XK_KP_Delete = 0xFF9F
XK_KP_Equal = 0xFFBD
XK_KP_Multiply = 0xFFAA
XK_KP_Add = 0xFFAB
XK_KP_Separator = 0xFFAC
XK_KP_Subtract = 0xFFAD
XK_KP_Decimal = 0xFFAE
XK_KP_Divide = 0xFFAF
XK_KP_0 = 0xFFB0
XK_KP_1 = 0xFFB1
XK_KP_2 = 0xFFB2
XK_KP_3 = 0xFFB3
XK_KP_4 = 0xFFB4
XK_KP_5 = 0xFFB5
XK_KP_6 = 0xFFB6
XK_KP_7 = 0xFFB7
XK_KP_8 = 0xFFB8
XK_KP_9 = 0xFFB9
XK_F1 = 0xFFBE
XK_F2 = 0xFFBF
XK_F3 = 0xFFC0
XK_F4 = 0xFFC1
XK_F5 = 0xFFC2
XK_F6 = 0xFFC3
XK_F7 = 0xFFC4
XK_F8 = 0xFFC5
XK_F9 = 0xFFC6
XK_F10 = 0xFFC7
XK_F11 = 0xFFC8
XK_L1 = 0xFFC8
XK_F12 = 0xFFC9
XK_L2 = 0xFFC9
XK_F13 = 0xFFCA
XK_L3 = 0xFFCA
XK_F14 = 0xFFCB
XK_L4 = 0xFFCB
XK_F15 = 0xFFCC
XK_L5 = 0xFFCC
XK_F16 = 0xFFCD
XK_L6 = 0xFFCD
XK_F17 = 0xFFCE
XK_L7 = 0xFFCE
XK_F18 = 0xFFCF
XK_L8 = 0xFFCF
XK_F19 = 0xFFD0
XK_L9 = 0xFFD0
XK_F20 = 0xFFD1
XK_L10 = 0xFFD1
XK_F21 = 0xFFD2
XK_R1 = 0xFFD2
XK_F22 = 0xFFD3
XK_R2 = 0xFFD3
XK_F23 = 0xFFD4
XK_R3 = 0xFFD4
XK_F24 = 0xFFD5
XK_R4 = 0xFFD5
XK_F25 = 0xFFD6
XK_R5 = 0xFFD6
XK_F26 = 0xFFD7
XK_R6 = 0xFFD7
XK_F27 = 0xFFD8
XK_R7 = 0xFFD8
XK_F28 = 0xFFD9
XK_R8 = 0xFFD9
XK_F29 = 0xFFDA
XK_R9 = 0xFFDA
XK_F30 = 0xFFDB
XK_R10 = 0xFFDB
XK_F31 = 0xFFDC
XK_R11 = 0xFFDC
XK_F32 = 0xFFDD
XK_R12 = 0xFFDD
XK_F33 = 0xFFDE
XK_R13 = 0xFFDE
XK_F34 = 0xFFDF
XK_R14 = 0xFFDF
XK_F35 = 0xFFE0
XK_R15 = 0xFFE0
XK_Shift_L = 0xFFE1
XK_Shift_R = 0xFFE2
XK_Control_L = 0xFFE3
XK_Control_R = 0xFFE4
XK_Caps_Lock = 0xFFE5
XK_Shift_Lock = 0xFFE6
XK_Meta_L = 0xFFE7
XK_Meta_R = 0xFFE8
XK_Alt_L = 0xFFE9
XK_Alt_R = 0xFFEA
XK_Super_L = 0xFFEB
XK_Super_R = 0xFFEC
XK_Hyper_L = 0xFFED
XK_Hyper_R = 0xFFEE
