XK_emspace = 0xaa1
XK_enspace = 0xaa2
XK_em3space = 0xaa3
XK_em4space = 0xaa4
XK_digitspace = 0xaa5
XK_punctspace = 0xaa6
XK_thinspace = 0xaa7
XK_hairspace = 0xaa8
XK_emdash = 0xaa9
XK_endash = 0xaaa
XK_signifblank = 0xaac
XK_ellipsis = 0xaae
XK_doubbaselinedot = 0xaaf
XK_onethird = 0xab0
XK_twothirds = 0xab1
XK_onefifth = 0xab2
XK_twofifths = 0xab3
XK_threefifths = 0xab4
XK_fourfifths = 0xab5
XK_onesixth = 0xab6
XK_fivesixths = 0xab7
XK_careof = 0xab8
XK_figdash = 0xabb
XK_leftanglebracket = 0xabc
XK_decimalpoint = 0xabd
XK_rightanglebracket = 0xabe
XK_marker = 0xabf
XK_oneeighth = 0xac3
XK_threeeighths = 0xac4
XK_fiveeighths = 0xac5
XK_seveneighths = 0xac6
XK_trademark = 0xac9
XK_signaturemark = 0xaca
XK_trademarkincircle = 0xacb
XK_leftopentriangle = 0xacc
XK_rightopentriangle = 0xacd
XK_emopencircle = 0xace
XK_emopenrectangle = 0xacf
XK_leftsinglequotemark = 0xad0
XK_rightsinglequotemark = 0xad1
XK_leftdoublequotemark = 0xad2
XK_rightdoublequotemark = 0xad3
XK_prescription = 0xad4
XK_minutes = 0xad6
XK_seconds = 0xad7
XK_latincross = 0xad9
XK_hexagram = 0xada
XK_filledrectbullet = 0xadb
XK_filledlefttribullet = 0xadc
XK_filledrighttribullet = 0xadd
XK_emfilledcircle = 0xade
XK_emfilledrect = 0xadf
XK_enopencircbullet = 0xae0
XK_enopensquarebullet = 0xae1
XK_openrectbullet = 0xae2
XK_opentribulletup = 0xae3
XK_opentribulletdown = 0xae4
XK_openstar = 0xae5
XK_enfilledcircbullet = 0xae6
XK_enfilledsqbullet = 0xae7
XK_filledtribulletup = 0xae8
XK_filledtribulletdown = 0xae9
XK_leftpointer = 0xaea
XK_rightpointer = 0xaeb
XK_club = 0xaec
XK_diamond = 0xaed
XK_heart = 0xaee
XK_maltesecross = 0xaf0
XK_dagger = 0xaf1
XK_doubledagger = 0xaf2
XK_checkmark = 0xaf3
XK_ballotcross = 0xaf4
XK_musicalsharp = 0xaf5
XK_musicalflat = 0xaf6
XK_malesymbol = 0xaf7
XK_femalesymbol = 0xaf8
XK_telephone = 0xaf9
XK_telephonerecorder = 0xafa
XK_phonographcopyright = 0xafb
XK_caret = 0xafc
XK_singlelowquotemark = 0xafd
XK_doublelowquotemark = 0xafe
XK_cursor = 0xaff
