XK_XF86_MonBrightnessUp   	= 0x1008FF02  
XK_XF86_MonBrightnessDown 	= 0x1008FF03  
XK_XF86_KbdLightOnOff     	= 0x1008FF04  
XK_XF86_KbdBrightnessUp   	= 0x1008FF05  
XK_XF86_KbdBrightnessDown 	= 0x1008FF06  

XK_XF86_Standby			= 0x1008FF10   
XK_XF86_AudioLowerVolume	= 0x1008FF11   
XK_XF86_AudioMute		= 0x1008FF12   
XK_XF86_AudioRaiseVolume	= 0x1008FF13   
XK_XF86_AudioPlay		= 0x1008FF14   
XK_XF86_AudioStop		= 0x1008FF15   
XK_XF86_AudioPrev		= 0x1008FF16   
XK_XF86_AudioNext		= 0x1008FF17   
XK_XF86_HomePage		= 0x1008FF18   
XK_XF86_Mail			= 0x1008FF19   
XK_XF86_Start			= 0x1008FF1A   
XK_XF86_Search			= 0x1008FF1B   
XK_XF86_AudioRecord		= 0x1008FF1C   

XK_XF86_Calculator		= 0x1008FF1D   
XK_XF86_Memo			= 0x1008FF1E   
XK_XF86_ToDoList		= 0x1008FF1F   
XK_XF86_Calendar		= 0x1008FF20   
XK_XF86_PowerDown		= 0x1008FF21   
XK_XF86_ContrastAdjust		= 0x1008FF22   
XK_XF86_RockerUp		= 0x1008FF23   
XK_XF86_RockerDown		= 0x1008FF24   
XK_XF86_RockerEnter		= 0x1008FF25   

XK_XF86_Back			= 0x1008FF26   
XK_XF86_Forward			= 0x1008FF27   
XK_XF86_Stop			= 0x1008FF28   
XK_XF86_Refresh			= 0x1008FF29   
XK_XF86_PowerOff		= 0x1008FF2A   
XK_XF86_WakeUp			= 0x1008FF2B   
XK_XF86_Eject           	= 0x1008FF2C   
XK_XF86_ScreenSaver     	= 0x1008FF2D   
XK_XF86_WWW             	= 0x1008FF2E   
XK_XF86_Sleep           	= 0x1008FF2F   
XK_XF86_Favorites		= 0x1008FF30   
XK_XF86_AudioPause		= 0x1008FF31   
XK_XF86_AudioMedia		= 0x1008FF32   
XK_XF86_MyComputer		= 0x1008FF33   
XK_XF86_VendorHome		= 0x1008FF34   
XK_XF86_LightBulb		= 0x1008FF35   
XK_XF86_Shop			= 0x1008FF36   
XK_XF86_History			= 0x1008FF37   
XK_XF86_OpenURL			= 0x1008FF38   
XK_XF86_AddFavorite		= 0x1008FF39   
XK_XF86_HotLinks		= 0x1008FF3A   
XK_XF86_BrightnessAdjust	= 0x1008FF3B   
XK_XF86_Finance			= 0x1008FF3C   
XK_XF86_Community		= 0x1008FF3D   
XK_XF86_AudioRewind		= 0x1008FF3E   
XK_XF86_XF86BackForward		= 0x1008FF3F   
XK_XF86_Launch0			= 0x1008FF40   
XK_XF86_Launch1			= 0x1008FF41   
XK_XF86_Launch2			= 0x1008FF42   
XK_XF86_Launch3			= 0x1008FF43   
XK_XF86_Launch4			= 0x1008FF44   
XK_XF86_Launch5			= 0x1008FF45   
XK_XF86_Launch6			= 0x1008FF46   
XK_XF86_Launch7			= 0x1008FF47   
XK_XF86_Launch8			= 0x1008FF48   
XK_XF86_Launch9			= 0x1008FF49   
XK_XF86_LaunchA			= 0x1008FF4A   
XK_XF86_LaunchB			= 0x1008FF4B   
XK_XF86_LaunchC			= 0x1008FF4C   
XK_XF86_LaunchD			= 0x1008FF4D   
XK_XF86_LaunchE			= 0x1008FF4E   
XK_XF86_LaunchF			= 0x1008FF4F   

XK_XF86_ApplicationLeft		= 0x1008FF50   
XK_XF86_ApplicationRight	= 0x1008FF51   
XK_XF86_Book			= 0x1008FF52   
XK_XF86_CD			= 0x1008FF53   
XK_XF86_Calculater		= 0x1008FF54   
XK_XF86_Clear			= 0x1008FF55   
XK_XF86_Close			= 0x1008FF56   
XK_XF86_Copy			= 0x1008FF57   
XK_XF86_Cut			= 0x1008FF58   
XK_XF86_Display			= 0x1008FF59   
XK_XF86_DOS			= 0x1008FF5A   
XK_XF86_Documents		= 0x1008FF5B   
XK_XF86_Excel			= 0x1008FF5C   
XK_XF86_Explorer		= 0x1008FF5D   
XK_XF86_Game			= 0x1008FF5E   
XK_XF86_Go			= 0x1008FF5F   
XK_XF86_iTouch			= 0x1008FF60   
XK_XF86_LogOff			= 0x1008FF61   
XK_XF86_Market			= 0x1008FF62   
XK_XF86_Meeting			= 0x1008FF63   
XK_XF86_MenuKB			= 0x1008FF65   
XK_XF86_MenuPB			= 0x1008FF66   
XK_XF86_MySites			= 0x1008FF67   
XK_XF86_New			= 0x1008FF68   
XK_XF86_News			= 0x1008FF69   
XK_XF86_OfficeHome		= 0x1008FF6A   
XK_XF86_Open			= 0x1008FF6B   
XK_XF86_Option			= 0x1008FF6C   
XK_XF86_Paste			= 0x1008FF6D   
XK_XF86_Phone			= 0x1008FF6E   
XK_XF86_Q			= 0x1008FF70   
XK_XF86_Reply			= 0x1008FF72   
XK_XF86_Reload			= 0x1008FF73   
XK_XF86_RotateWindows		= 0x1008FF74   
XK_XF86_RotationPB		= 0x1008FF75   
XK_XF86_RotationKB		= 0x1008FF76   
XK_XF86_Save			= 0x1008FF77   
XK_XF86_ScrollUp		= 0x1008FF78   
XK_XF86_ScrollDown		= 0x1008FF79   
XK_XF86_ScrollClick		= 0x1008FF7A   
XK_XF86_Send			= 0x1008FF7B   
XK_XF86_Spell			= 0x1008FF7C   
XK_XF86_SplitScreen		= 0x1008FF7D   
XK_XF86_Support			= 0x1008FF7E   
XK_XF86_TaskPane		= 0x1008FF7F   
XK_XF86_Terminal		= 0x1008FF80   
XK_XF86_Tools			= 0x1008FF81   
XK_XF86_Travel			= 0x1008FF82   
XK_XF86_UserPB			= 0x1008FF84   
XK_XF86_User1KB			= 0x1008FF85   
XK_XF86_User2KB			= 0x1008FF86   
XK_XF86_Video			= 0x1008FF87   
XK_XF86_WheelButton		= 0x1008FF88   
XK_XF86_Word			= 0x1008FF89   
XK_XF86_Xfer			= 0x1008FF8A
XK_XF86_ZoomIn			= 0x1008FF8B   
XK_XF86_ZoomOut			= 0x1008FF8C   
                        	
XK_XF86_Away			= 0x1008FF8D   
XK_XF86_Messenger		= 0x1008FF8E   
XK_XF86_WebCam			= 0x1008FF8F   
XK_XF86_MailForward		= 0x1008FF90   
XK_XF86_Pictures		= 0x1008FF91   
XK_XF86_Music			= 0x1008FF92   
                        	
XK_XF86_Battery			= 0x1008FF93   
XK_XF86_Bluetooth		= 0x1008FF94   
XK_XF86_WLAN			= 0x1008FF95   
XK_XF86_UWB			= 0x1008FF96   
                        	
XK_XF86_AudioForward		= 0x1008FF97   
XK_XF86_AudioRepeat		= 0x1008FF98   
XK_XF86_AudioRandomPlay		= 0x1008FF99   
XK_XF86_Subtitle		= 0x1008FF9A   
XK_XF86_AudioCycleTrack		= 0x1008FF9B   
XK_XF86_CycleAngle		= 0x1008FF9C   
XK_XF86_FrameBack		= 0x1008FF9D   
XK_XF86_FrameForward		= 0x1008FF9E   
XK_XF86_Time			= 0x1008FF9F   
XK_XF86_Select			= 0x1008FFA0   
XK_XF86_View			= 0x1008FFA1   
XK_XF86_TopMenu			= 0x1008FFA2   
                        	
XK_XF86_Red			= 0x1008FFA3   
XK_XF86_Green			= 0x1008FFA4   
XK_XF86_Yellow			= 0x1008FFA5   
XK_XF86_Blue            	= 0x1008FFA6   
                        	
XK_XF86_Switch_VT_1		= 0x1008FE01
XK_XF86_Switch_VT_2		= 0x1008FE02
XK_XF86_Switch_VT_3		= 0x1008FE03
XK_XF86_Switch_VT_4		= 0x1008FE04
XK_XF86_Switch_VT_5		= 0x1008FE05
XK_XF86_Switch_VT_6		= 0x1008FE06
XK_XF86_Switch_VT_7		= 0x1008FE07
XK_XF86_Switch_VT_8		= 0x1008FE08
XK_XF86_Switch_VT_9		= 0x1008FE09
XK_XF86_Switch_VT_10		= 0x1008FE0A
XK_XF86_Switch_VT_11		= 0x1008FE0B
XK_XF86_Switch_VT_12		= 0x1008FE0C
                        	
XK_XF86_Ungrab			= 0x1008FE20   
XK_XF86_ClearGrab		= 0x1008FE21   
XK_XF86_Next_VMode		= 0x1008FE22   
XK_XF86_Prev_VMode		= 0x1008FE23

