XK_Thai_kokai = 0xda1
XK_Thai_khokhai = 0xda2
XK_Thai_khokhuat = 0xda3
XK_Thai_khokhwai = 0xda4
XK_Thai_khokhon = 0xda5
XK_Thai_khorakhang = 0xda6
XK_Thai_ngongu = 0xda7
XK_Thai_chochan = 0xda8
XK_Thai_choching = 0xda9
XK_Thai_chochang = 0xdaa
XK_Thai_soso = 0xdab
XK_Thai_chochoe = 0xdac
XK_Thai_yoying = 0xdad
XK_Thai_dochada = 0xdae
XK_Thai_topatak = 0xdaf
XK_Thai_thothan = 0xdb0
XK_Thai_thonangmontho = 0xdb1
XK_Thai_thophuthao = 0xdb2
XK_Thai_nonen = 0xdb3
XK_Thai_dodek = 0xdb4
XK_Thai_totao = 0xdb5
XK_Thai_thothung = 0xdb6
XK_Thai_thothahan = 0xdb7
XK_Thai_thothong = 0xdb8
XK_Thai_nonu = 0xdb9
XK_Thai_bobaimai = 0xdba
XK_Thai_popla = 0xdbb
XK_Thai_phophung = 0xdbc
XK_Thai_fofa = 0xdbd
XK_Thai_phophan = 0xdbe
XK_Thai_fofan = 0xdbf
XK_Thai_phosamphao = 0xdc0
XK_Thai_moma = 0xdc1
XK_Thai_yoyak = 0xdc2
XK_Thai_rorua = 0xdc3
XK_Thai_ru = 0xdc4
XK_Thai_loling = 0xdc5
XK_Thai_lu = 0xdc6
XK_Thai_wowaen = 0xdc7
XK_Thai_sosala = 0xdc8
XK_Thai_sorusi = 0xdc9
XK_Thai_sosua = 0xdca
XK_Thai_hohip = 0xdcb
XK_Thai_lochula = 0xdcc
XK_Thai_oang = 0xdcd
XK_Thai_honokhuk = 0xdce
XK_Thai_paiyannoi = 0xdcf
XK_Thai_saraa = 0xdd0
XK_Thai_maihanakat = 0xdd1
XK_Thai_saraaa = 0xdd2
XK_Thai_saraam = 0xdd3
XK_Thai_sarai = 0xdd4
XK_Thai_saraii = 0xdd5
XK_Thai_saraue = 0xdd6
XK_Thai_sarauee = 0xdd7
XK_Thai_sarau = 0xdd8
XK_Thai_sarauu = 0xdd9
XK_Thai_phinthu = 0xdda
XK_Thai_maihanakat_maitho = 0xdde
XK_Thai_baht = 0xddf
XK_Thai_sarae = 0xde0
XK_Thai_saraae = 0xde1
XK_Thai_sarao = 0xde2
XK_Thai_saraaimaimuan = 0xde3
XK_Thai_saraaimaimalai = 0xde4
XK_Thai_lakkhangyao = 0xde5
XK_Thai_maiyamok = 0xde6
XK_Thai_maitaikhu = 0xde7
XK_Thai_maiek = 0xde8
XK_Thai_maitho = 0xde9
XK_Thai_maitri = 0xdea
XK_Thai_maichattawa = 0xdeb
XK_Thai_thanthakhat = 0xdec
XK_Thai_nikhahit = 0xded
XK_Thai_leksun = 0xdf0
XK_Thai_leknung = 0xdf1
XK_Thai_leksong = 0xdf2
XK_Thai_leksam = 0xdf3
XK_Thai_leksi = 0xdf4
XK_Thai_lekha = 0xdf5
XK_Thai_lekhok = 0xdf6
XK_Thai_lekchet = 0xdf7
XK_Thai_lekpaet = 0xdf8
XK_Thai_lekkao = 0xdf9
