# Music Player Settings

mediaFileExtensions = ['.flac', '.mp3', '.ogg', '.wav'] # use lower case only
restartFilename = '.mp-restart'
nowPlayingFilename = '~/.nowplaying'
separator = '### skip the following songs ###'
version = "1.3.1"
date = "2016-06-17"
skipSongThatWasPlayingWhenLastKilled = True
