XK_Hangul = 0xff31
XK_Hangul_Start = 0xff32
XK_Hangul_End = 0xff33
XK_Hangul_Hanja = 0xff34
XK_Hangul_Jamo = 0xff35
XK_Hangul_Romaja = 0xff36
XK_Hangul_Codeinput = 0xff37
XK_Hangul_Jeonja = 0xff38
XK_Hangul_Banja = 0xff39
XK_Hangul_PreHanja = 0xff3a
XK_Hangul_PostHanja = 0xff3b
XK_Hangul_SingleCandidate = 0xff3c
XK_Hangul_MultipleCandidate = 0xff3d
XK_Hangul_PreviousCandidate = 0xff3e
XK_Hangul_Special = 0xff3f
XK_Hangul_switch = 0xFF7E
XK_Hangul_Kiyeog = 0xea1
XK_Hangul_SsangKiyeog = 0xea2
XK_Hangul_KiyeogSios = 0xea3
XK_Hangul_Nieun = 0xea4
XK_Hangul_NieunJieuj = 0xea5
XK_Hangul_NieunHieuh = 0xea6
XK_Hangul_Dikeud = 0xea7
XK_Hangul_SsangDikeud = 0xea8
XK_Hangul_Rieul = 0xea9
XK_Hangul_RieulKiyeog = 0xeaa
XK_Hangul_RieulMieum = 0xeab
XK_Hangul_RieulPieub = 0xeac
XK_Hangul_RieulSios = 0xead
XK_Hangul_RieulTieut = 0xeae
XK_Hangul_RieulPhieuf = 0xeaf
XK_Hangul_RieulHieuh = 0xeb0
XK_Hangul_Mieum = 0xeb1
XK_Hangul_Pieub = 0xeb2
XK_Hangul_SsangPieub = 0xeb3
XK_Hangul_PieubSios = 0xeb4
XK_Hangul_Sios = 0xeb5
XK_Hangul_SsangSios = 0xeb6
XK_Hangul_Ieung = 0xeb7
XK_Hangul_Jieuj = 0xeb8
XK_Hangul_SsangJieuj = 0xeb9
XK_Hangul_Cieuc = 0xeba
XK_Hangul_Khieuq = 0xebb
XK_Hangul_Tieut = 0xebc
XK_Hangul_Phieuf = 0xebd
XK_Hangul_Hieuh = 0xebe
XK_Hangul_A = 0xebf
XK_Hangul_AE = 0xec0
XK_Hangul_YA = 0xec1
XK_Hangul_YAE = 0xec2
XK_Hangul_EO = 0xec3
XK_Hangul_E = 0xec4
XK_Hangul_YEO = 0xec5
XK_Hangul_YE = 0xec6
XK_Hangul_O = 0xec7
XK_Hangul_WA = 0xec8
XK_Hangul_WAE = 0xec9
XK_Hangul_OE = 0xeca
XK_Hangul_YO = 0xecb
XK_Hangul_U = 0xecc
XK_Hangul_WEO = 0xecd
XK_Hangul_WE = 0xece
XK_Hangul_WI = 0xecf
XK_Hangul_YU = 0xed0
XK_Hangul_EU = 0xed1
XK_Hangul_YI = 0xed2
XK_Hangul_I = 0xed3
XK_Hangul_J_Kiyeog = 0xed4
XK_Hangul_J_SsangKiyeog = 0xed5
XK_Hangul_J_KiyeogSios = 0xed6
XK_Hangul_J_Nieun = 0xed7
XK_Hangul_J_NieunJieuj = 0xed8
XK_Hangul_J_NieunHieuh = 0xed9
XK_Hangul_J_Dikeud = 0xeda
XK_Hangul_J_Rieul = 0xedb
XK_Hangul_J_RieulKiyeog = 0xedc
XK_Hangul_J_RieulMieum = 0xedd
XK_Hangul_J_RieulPieub = 0xede
XK_Hangul_J_RieulSios = 0xedf
XK_Hangul_J_RieulTieut = 0xee0
XK_Hangul_J_RieulPhieuf = 0xee1
XK_Hangul_J_RieulHieuh = 0xee2
XK_Hangul_J_Mieum = 0xee3
XK_Hangul_J_Pieub = 0xee4
XK_Hangul_J_PieubSios = 0xee5
XK_Hangul_J_Sios = 0xee6
XK_Hangul_J_SsangSios = 0xee7
XK_Hangul_J_Ieung = 0xee8
XK_Hangul_J_Jieuj = 0xee9
XK_Hangul_J_Cieuc = 0xeea
XK_Hangul_J_Khieuq = 0xeeb
XK_Hangul_J_Tieut = 0xeec
XK_Hangul_J_Phieuf = 0xeed
XK_Hangul_J_Hieuh = 0xeee
XK_Hangul_RieulYeorinHieuh = 0xeef
XK_Hangul_SunkyeongeumMieum = 0xef0
XK_Hangul_SunkyeongeumPieub = 0xef1
XK_Hangul_PanSios = 0xef2
XK_Hangul_KkogjiDalrinIeung = 0xef3
XK_Hangul_SunkyeongeumPhieuf = 0xef4
XK_Hangul_YeorinHieuh = 0xef5
XK_Hangul_AraeA = 0xef6
XK_Hangul_AraeAE = 0xef7
XK_Hangul_J_PanSios = 0xef8
XK_Hangul_J_KkogjiDalrinIeung = 0xef9
XK_Hangul_J_YeorinHieuh = 0xefa
XK_Korean_Won = 0xeff
